#coding=utf-8
from .base import *
"""
返回Node
"""
class BaseFormat:
    def init(self):
        return self
    def deal(self, data, fc, **params):
        return None
    def __call__(self, data, fc, **params):
        return self.deal(data, fc, **params)

pass


class FormatNode:
    def str(self):
        if self.is_leaf():
            return "<Leaf={val}>".format(val = self.value)
        else:
            cs = [str(c) for c in self.childs]
            return "<Node:{val}>".format(val = ", ".join(cs))
    def __str__(self):
        return self.str()
    def __repr__(self):
        return str(self)
    def __init__(self):
        self.init()
    def init(self):
        self.to_top = 0
        self.max_depth = 0
        self.childs = []
        self.value = None
        self.up = None
        return self
    def update_top(self, up):
        self.to_top =up.to_top+1
        if self.is_leaf():
            return
        for _node in self.childs:
            _node.update_top(self)
    def update(self, node):
        up = self
        while up is not None:
            depth = node.max_depth +1
            if depth>up.max_depth:
                up.max_depth = depth
                node, up = up, up.up
            else:
                break
    def add(self, node):
        node.update_top(self)
        self.update(node)
        self.childs.append(node)
        node.up = self
        return self
    def val(self, value):
        self.value = value
        return self
    def is_leaf(self):
        return len(self.childs)==0

pass

class SymbolFormatNode(FormatNode):
    def str(self):
        return "<symbol {key}>".format(key = self.value)
    pass

pass
class KVFormatNode(FormatNode):
    def str(self):
        return "<key-value>"
    pass

pass
class SptFormatNode(FormatNode):
    def str(self):
        return "<split>"

pass

class SpcFormatNode(FormatNode):
    def __init__(self, add_depth = 0, add_top = 0):
        super(SpcFormatNode, self).__init__()
        self._depth = add_depth
        self._top = add_top
    def depth(self):
        return max(0, self._depth+self.up.max_depth)
    def top(self):
        return max(0, self._top+self.up.to_top)
    def str(self):
        return "<space>{i},{j}".format(i=self.up.max_depth, j = self.up.to_top)

pass

class SymbolFormat(BaseFormat):
    def deal(self, node, fc, **params):
        if not isinstance(node, SymbolFormatNode):
            return None
        return node.value

pass
class KeyFormat(BaseFormat):
    def __init__(self, key, node_type):
        self.key = key
        self.node_type = node_type
    def deal(self, node, fc, **params):
        if not isinstance(node, self.node_type):
            return None
        return self.key

pass

class SpcFormat(BaseFormat):
    def __init__(self, spc=" ", enter = "\n", size = 4, format = False, simple = 1, simple_size = 1):
        self.format = format
        self.simple = simple
        self.size = size
        self.enter = enter
        self.simple_size = simple_size
        self.spc = spc*size
        self._spc = spc*simple_size
        pass
    def deal(self, node, fc, **params):
        if not isinstance(node, SpcFormatNode):
            return None
        if not self.format:
            return self._spc
        max_depth = node.depth()
        if max_depth<=self.simple:
            return self._spc
        return self.enter+(self.spc*node.top())


pass


class NodeFormat(BaseFormat):
    def deal(self, node, fc, **params):
        if node.is_leaf():
            return None
        rst = None
        for _node in node.childs:
            _rst = fc(_node, **params)
            if _rst is None:
                raise Exception("Unregist deal node:"+str(_node))
            if rst is None:
                rst = _rst
            else:
                rst += _rst
        return rst

pass


#from buildz.confz import read, loadfile, output, fread
from .build import Builder, main
from .keys import help
"""
json-like(called 'confz' in here) data format function:
    read: string -> obj
    output: obj -> string
    fread(filepath, coding="utf-8"): filepath -> string 
        #open(filepath, 'rb').read().decode(coding)
    loadfile(filepath, coding="utf-8"): filepath -> obj 
        #equals to read(fread(filepath, coding))

a framework to build code:
    function: main
    Object: Builder
    Description:
        build instance and run from confz format configure file, here is an example in buildz/demo
        从confz格式文件配置中生成对象实例和运行，例子在buildz/demo里

        you can open an commond line in demo folder and run like this:
        测试：在demo文件夹中打开命令行，运行：

            1) python test.py ./demo.confz ./value.confz
            or:
            2) python test.py ./run.confz

        1) and 2) is equivalent
        1)和2)是等价的
"""


#
from ..ioc.base import Base, EncapeData
from .base import FormatData,BaseDeal
from buildz import xf, pyz
import os
dp = os.path.dirname(__file__)
join = os.path.join
class ObjectDeal(BaseDeal):
    """
    {
        id: ...
        type: object
        single: 1
        source: 
        construct: {
            args: []
            maps: {
            }
        }
        sets: [
            {key: ..., val: {...}}
        ]
    }
    """
    def init(self, fp_lists = None, fp_defaults = None, fp_cst = None, fp_set = None):
        self.singles = {}
        self.sources = {}
        super().init("ObjectDeal", fp_lists, fp_defaults, 
            join(dp, "conf", "obj_lists.js"),
            join(dp, "conf", "obj_defaults.js"))
        if fp_set is None:
            fp_set = join(dp, 'conf', 'obj_set_lists.js')
        if fp_cst is None:
            fp_cst = join(dp, 'conf', 'obj_cst_lists.js')
        self.fmt_set = FormatData(xf.loads(xf.fread(fp_set)))
        self.fmt_cst = FormatData(xf.loads(xf.fread(fp_cst)))
    def get_maps(self, maps, sid, id):
        if id is None:
            return None
        if sid not in maps:
            return None
        maps = maps[sid]
        if id not in maps:
            return None
        return maps[id]
    def set_maps(self, maps, sid, id, obj):
        if sid not in maps:
            maps[sid] = {}
        maps[sid][id] = obj
    def deal(self, edata:EncapeData):
        sid = edata.sid
        data = edata.data
        data = self.format(data)
        id = xf.g(data, id = None)
        single = xf.g(data, single=None)
        if id is None:
            single = 0
        if single is None:
            single = 1
        if single:
            obj = self.get_maps(self.singles, sid, id)
            if obj is not None:
                return obj
        conf = edata.conf
        confs = edata.confs
        source = xf.g(data, source=0)
        fc = xf.get(self.sources, source, None)
        if fc is None:
            fc = pyz.load(source)
            self.sources[source]=fc
        cst = xf.g(data, construct = [])
        cst = self.fmt_cst(cst)
        args = xf.g(cst, args=[])
        maps = xf.g(cst, maps={})
        args = [self.get_obj(v, conf) for v in args]
        maps = {k:self.get_obj(maps[k], conf) for k in maps}
        obj = fc(*args, **maps)
        if single:
            self.set_maps(self.singles, sid, id, obj)
        sets = xf.g(data, sets=[])
        for kv in sets:
            kv = self.fmt_set(kv)
            k = kv['key']
            v = kv['val']
            v = self.get_obj(v, conf, obj)
            setattr(obj, k, v)
        calls = xf.g(data, calls=[])
        for call in calls:
            rst = self.get_obj(call, conf, obj)
        return obj

pass



#
from ..ioc.base import Base, EncapeData
from .base import FormatData,BaseDeal
from buildz import xf, pyz
import os
dp = os.path.dirname(__file__)
join = os.path.join
class MethodCallDeal(BaseDeal):
    """
    {
        id: ...
        type: mcall
        source: 
        method: 
        args: []
        maps: {}
    }
    [[id, mcall], source, method, args, maps]
    """
    def init(self, fp_lists = None, fp_defaults = None):
        self.singles = {}
        self.sources = {}
        super().init("MethodCallDeal", fp_lists, fp_defaults, 
            join(dp, "conf", "mcall_lists.js"),
            join(dp, "conf", "mcall_defaults.js"))
    def deal(self, edata:EncapeData):
        sid = edata.sid
        data = edata.data
        conf = edata.conf
        data = self.format(data)
        src = edata.src
        source = xf.g(data, source=None)
        method = xf.g(data, method=0)
        if source is not None:
            source = conf.get(source)
        if source is None:
            source = src
        if source is None:
            raise Exception(f"not object for method {method}")
        if src is None:
            src = source
        method = getattr(source, method)
        args = xf.g(data, args=[])
        maps = xf.g(data, maps ={})
        args = [self.get_obj(v, conf, src) for v in args]
        maps = {k:self.get_obj(maps[k], conf, src) for k in maps}
        return method(*args, **maps)

pass



#coding=utf-8


"""
impl in class
"""

import io
# io.StringIO("asdf")
from .buff import *
from .fc import *
from .fmt_base import *

class Confz:
    def fcs(self, fc, *args, **maps):
        return fc(*self.cs(*args), **maps)
    def add_fc(self, fc, *args, **maps):
        obj = self.fcs(fc, *args, **maps)
        self.add(obj)
        return self
    def add_top_fc(self, fc, *args, **maps):
        obj = self.fcs(fc, *args, **maps)
        self.add_top(obj)
        return self
    def cs(self, *args):
        return [self.c(k) for k in args]
    def c(self, s):
        if self.bts:
            s = s.encode("utf-8")
        return s
    def add(self,obj):
        obj.init(self.reg)
        self.prevs.append(obj.prev)
        self.deals.append(obj.deal)
        return self
    def add_top(self, obj):
        obj.init(self.reg)
        self.prevs=[obj.prev]+self.prevs
        self.deals=[obj.deal]+self.deals
        return self
    def do(self, fcs, buff, stack):
        for fc in fcs:
            if fc(buff, stack):
                #print("fc:", fc)
                return True
        return False
    def __init__(self, bts= False):
        self.bts= bts
        self.deals = []
        self.prevs = []
        self.reg = Reg()
    def load(self, reader):
        buff = BufferImpl(reader, "" if not self.bts else b"")
        queue = []
        while self.do(self.prevs, buff, queue):
            pass
        #print("queue:", [it.val for it in queue])
        stack = []
        while self.do(self.deals, queue, stack):
            #print("QUEUE:", queue)
            #print("STACK:", stack)
            #print("*"*30)
            pass
        if len(stack)==0:
            raise Exception("ERROR not data")
        if len(stack)==1:
            return stack[0].val
        else:
            return [it.val for it in stack]

pass

class Map:
    def __init__(self):
        self.maps = {}
    def set(self, key, val):
        self.maps[key] = val
    def __call__(self, s):
        if s not in self.maps:
            raise Exception("Error key:"+s)
        return self.maps[s]

pass
def get_none(s):
    return None

pass


def build(bts = False):
    cfz = Confz(bts)
    cfz.add_fc(SpcDeal)

    cfz.add_fc(SetDeal, ":").add_fc(SetDeal,"=")
    types = TypesDeal()
    types.add_deal(cfz.fcs(TypeDealB, "|"))
    types.add_deal(cfz.fcs(TypeDealA, "<", ">"))

    types.add_type("int", int)
    types.add_type("i", int)
    types.add_type("float", float)
    types.add_type("f", float)
    types.add_type("str", str)
    types.add_type("s", str)

    get_bool = Map()
    get_bool.set("true", True)
    get_bool.set("1", True)
    get_bool.set("false", False)
    get_bool.set("0", False)

    types.add_type("bool", get_bool)
    types.add_type("b", get_bool)
    types.add_type("null", get_none)
    types.add_type("nil", get_none)
    types.add_type("~", get_none)

    cfz.add(types)


    cfz.add_fc(StrDeal,"/*","*/", note = True)
    cfz.add_fc(StrDeal,*["###"]*2, note = True)
    cfz.add_fc(StrDeal,"#", "\n", single_line = True, note = True)
    cfz.add_fc(StrDeal,"//", "\n", single_line = True, note = True)
    #cfz.add_fc(StrDeal,*["'''"]*2)
    #cfz.add_fc(StrDeal,*['"""']*2)
    #cfz.add_fc(StrDeal,*["'"]*2, single_line = True)
    #cfz.add_fc(StrDeal,*['"']*2, single_line = True)


    cfz.add_fc(StrDeal,"r'''", "'''")
    cfz.add_fc(StrDeal,'r"""', '"""')

    cfz.add_fc(StrDeal,"r'", "'", single_line = True)
    cfz.add_fc(StrDeal,'r"', '"', single_line = True)

    cfz.add_fc(TranslateStrDeal,"'''", "'''")
    cfz.add_fc(TranslateStrDeal,'"""', '"""')

    cfz.add_fc(TranslateStrDeal,"'", "'", single_line = True)
    cfz.add_fc(TranslateStrDeal,'"', '"', single_line = True)
    
    cfz.add_fc(ItemDeal, ",").add_fc(ItemDeal, "\n").add_fc(ItemDeal, ";")
    cfz.add_fc(ListDeal, "[", "]")
    cfz.add_fc(ListDeal, "(", ")")
    cfz.add_fc(MapDeal, "{", "}")
    cfz.add_fc(NextCharDeal)
    return cfz

pass
import io
def loads(s):
    cfz = build(type(s)==bytes)
    return cfz.load(io.StringIO(s).read)

pass

from .file import *

def loadfile(fp, coding = 'utf-8'):
    return loads(fread(fp, coding))

pass


#coding=utf-8
from .base import *
from .fc_list import *
from .fc_set import *


"""
二元符<val, type>

"""

class TypeDealA(ListDeal):
    def __init__(self, left, right):
        super(TypeDealA, self).__init__(left, right)
        self._types = {}
    def types(self, maps):
        self._types = dict(maps)
    def deal(self, queue, stack):
        rst = super(TypeDealA, self).deal(queue, stack)
        if not rst:
            return False
        item = stack[-1]
        vals = item.val
        if Key.is_inst(vals):
            return True
        stack.pop(-1)
        if len(vals)!=2:
            raise FormatExp("type error: requires two elements", item.pos)
        val, _type = vals
        if _type not in self._types:
            raise FormatExp("type error: can't deal with type {type}".format(type = _type), item.pos)
        try:
            val = self._types[_type](val)
        except Exception as exp:
            raise FormatExp("type error: "+str(exp), item.pos)
        stack.append(Item(val, item.pos))
        return True

pass

"""
二元符val|type
"""
class TypeDealB(SetDeal):
    def __init__(self, *args):
        super(TypeDealB, self).__init__(*args)
        self._types = {}
    def types(self, maps):
        self._types = dict(maps)
    def deal(self, queue, stack):
        rst = super(TypeDealB, self).deal(queue, stack)
        if not rst:
            return False
        item = stack.pop(-1)
        vals = item.val
        val = vals.key
        _type = vals.val
        if _type not in self._types:
            raise FormatExp("type error: can't deal with type {type}".format(type = _type), item.pos)
        try:
            val = self._types[_type](val)
        except Exception as exp:
            raise FormatExp("type error: "+str(exp), item.pos)
        stack.append(Item(val, item.pos))
        return True

pass


class TypesDeal:
    def __init__(self):
        self.deals = []
        self.types = {}
    def add_deal(self, deal):
        self.deals.append(deal)
        return self
    def add_type(self, key, fc):
        self.types[key] = fc
        return self
    def add(self, obj, fc = None):
        if isinstance(obj, BaseDeal):
            return self.add_deal(obj)
        else:
            return self.add_type(obj, fc)
    def init(self, reg):
        for deal in self.deals:
            deal.types(self.types)
            deal.init(reg)
    def prev(self, buff, queue):
        for deal in self.deals:
            if deal.prev(buff, queue):
                return True
        return False
    def deal(self, queue, stack):
        for deal in self.deals:
            if deal.deal(queue, stack):
                return True
        return False

pass



#coding=utf-8

from .mg import *
from .mg_fmt import *
from .file import *

"""
loads(string) => object

dumps(object, format = False, simple = 1, as_json = False, t_single = True) => string
format: 是否格式化
as_json: 数值用json方法生成
t_single: 字符串中换行改成\n


loadfile(filepath, coding = 'utf-8') => object
= loads(fread(filepath, coding))

fread(filepath, coding = 'utf-8') => string
二进制读取文件并用coding编码，编码失败会换另一个编码尝试(gbk)
"""


#coding=utf-8
from .base import *

class TypeSet:
    def __str__(self):
        return "@@TypeSet"
    def __repr__(self):
        return str(self)

pass
class SetDeal(BaseDeal):
    Type = TypeSet()
    def __str__(self):
        return "@@SetDeal"
    def __repr__(self):
        return str(self)
    def find(self, buff):
        for spt in self.spts:
            if self.check_curr(buff, spt):
                return spt
        return None
    def __init__(self, *spts):
        if len(spts)==0:
            raise Exception("at least one symbol")
        self.spts= spts
    def init(self, reg):
        self.k_spts = [reg(k) for k in self.spts]
        self.k_spt = self.k_spts[0]
    def prev(self, buff, queue):
        find = self.find(buff)
        if find is None:
            return False
        lf = len(find)
        rm = buff.remain().strip()
        pos = buff.pos_remain()
        c_pos = buff.pos_curr()
        if len(queue)==0 or len(rm)>0:
            queue.append(Item(rm, pos))
        queue.append(Item(self.k_spt, c_pos, find, SetDeal.Type))
        buff.deal2curr(lf)
        return True
    def deal(self, queue, stack):
        if len(stack)<3:
            return False
        if self.k_spt(stack[-2].val):
            if len(queue)>0:
                val = queue[0]
                if val.type == SetDeal.Type:
                    return False
            val = stack[-1]
            if Key.is_inst(val.val):
                return False
            val = stack.pop(-1)
            tmp = stack.pop(-1)
            key = stack.pop(-1)
            kv = KeyVal(key.val,val.val)
            item = Item(kv, key.pos)
            stack.append(item)
            return True
        return False


#
from ..ioc.base import Base, EncapeData

class ValDeal(Base):
    """
    {
        val: ...
    }
    """
    def deal(self, edata:EncapeData):
        data = edata.data
        if type(data)==list:
            return data[-1]
        return data['data']

pass




from xf import loads
from loader import mg, buffer

from loader.deal import nextz, spt, strz, listz, spc, setz, mapz, reval
s = r"""test,'test string',
test enter
"    kong ge  "
[1,2,,3, " asdf ", hehe]1.23
{1:2,3:[4,5,6]}true,false,truefalse,null,None
#asdf
[]
"""
s=r"""
{
    a:b,
    test: ???
    id: 123
    cid: '123'
    content: '''hehe
    hehehe
    '''
    b:,#test
    a:[1,2,3,]
    v:{
        a:r"xx\n"
        b:"\n"
        c:d
    }
}
"""
def build_val(mgs):
    mgs.add(reval.ValDeal("[\+\-]?\d+", int))
    mgs.add(reval.ValDeal("[\+\-]?\d+\.\d+", float))
    mgs.add(reval.ValDeal("[\+\-]?\d+e[\+\-]?\d+", float))
    mgs.add(reval.ValDeal("null", lambda x:None))
    mgs.add(reval.ValDeal("true", lambda x:True))
    mgs.add(reval.ValDeal("false", lambda x:False))

pass
mgs = mg.Manager()
mgs.add(spc.PrevSpcDeal())
build_val(mgs)
mgs.add(strz.PrevStrDeal("r'''","'''",0,0,0))
mgs.add(strz.PrevStrDeal('r"""','"""',0,0,0))
mgs.add(strz.PrevStrDeal("r'","'",1,0,0))
mgs.add(strz.PrevStrDeal('r"','"',1,0,0))
mgs.add(strz.PrevStrDeal("###","###",0,1))
mgs.add(strz.PrevStrDeal("/*","*/",0,1))
mgs.add(strz.PrevStrDeal("'''","'''",0,0,1))
mgs.add(strz.PrevStrDeal('"""','"""',0,0,1))
mgs.add(strz.PrevStrDeal("#","\n",1,1))
mgs.add(strz.PrevStrDeal("//","\n",1,1))
mgs.add(strz.PrevStrDeal("'","'",1,0,1))
mgs.add(strz.PrevStrDeal('"','"',1,0,1))
mgs.add(setz.SetDeal(':'))
mgs.add(spt.PrevSptDeal(',',1))
mgs.add(spt.PrevSptDeal('\n'))
mgs.add(listz.ListDeal("[", "]"))
mgs.add(mapz.MapDeal("{", "}"))
mgs.add(nextz.PrevNextDeal())
input = buffer.BufferInput(s)
queue = mgs.load(input)
print(queue)
#for k in queue:
#    print(k)
#
#pass
print(mgs.pos.get())


#coding=utf-8

from buildz import xconfz as xf
print(xf.__path__)
s = "{a:}"
arr = [
    "[]",
    "{a:'',b:''}",
    "{a:''}",
    "{'':''}",
    "{a:'',b:''}"
]

for a in arr:
    print("data:", a)
    xf.loads(a)

pass



#coding=utf-8
from buildz import xf, argx, pyz, ioc, fz

class Deal:
    def deal(self):
        args, maps = argx.fetch()
        if len(args)<3:
            print("need params 1 to be filepath, params 2 to be data_id")
            return
        dp = args[1]
        fps = fz.search(dp, ".*\.js$")
        confs = ioc.build()
        confs.add_fps(fps)
        id = args[2]
        rst = confs.get(id)
        print(f"get {id}: {rst}")
        return rst

pass



s = r'''
[
    # single line note
    {
        filepath: "D:\demo\demo.txt"
        key: test.com
        array: [1, 2, 3]
    }
    ###
        multi-line note
        1
        2
        3
    ###
    {
        test: ":test{}??",
        val: <10, int>,
        cost: <10.0, float>,
        check: <true, bool>,
        <10, int>: "test"
    }
    #multi-line string
    """test line
        1
        2
        3
    """
    1
    2
    3
    # type not string
    <4, int>
    <5.0, float>
    <true, bool>
]
'''
s = r"""
[as/df, 'asdf', t est , 测试,'''
''',,]
{
    x:"\""
    a=0, #note 1
    b= 1;
    c= x;
    d= [a,b,c]y=0,
    y = 1,
    val = <1, i>




    bl = 1|b
    x = 0|nil
    lst = [
    a,b,c,

1
2
3


    x,y,z,,,
    ]
}
"""
s=r"""
[1,2,


3
4
5]
"""
try:
    from buildz import build
except:
    import sys
    import os
    sys.path.append(os.path.abspath("./../.."))
    from buildz import build
from buildz import xconfz as confz

import os
os.system("clear")
rst= confz.loads(s)
print(type(rst))
print(rst)
from buildz import confz as bk

print(bk.dumps(rst, format=True))



#
from ..ioc.base import Base, EncapeData
from .base import FormatData,BaseDeal
from buildz import xf, pyz
import os
dp = os.path.dirname(__file__)
join = os.path.join
class CallsDeal(BaseDeal):
    """
    {
        id: ...
        type: calls
        calls: [
            ...
        ]
    }
    [[id, type], calls]
    """
    def init(self, fp_lists = None, fp_defaults = None):
        self.singles = {}
        self.sources = {}
        super().init("CallsDeal", fp_lists, fp_defaults, 
            join(dp, "conf", "calls_lists.js"),
            join(dp, "conf", "calls_defaults.js"))
    def deal(self, edata:EncapeData):
        sid = edata.sid
        data = edata.data
        conf = edata.conf
        data = self.format(data)
        src = edata.src
        calls = xf.g(data, calls=[])
        rst = None
        for call in calls:
            rst = self.get_obj(call, conf, src)
        return rst

pass



#coding=utf-8
try:
    from buildz import build
except:
    import sys
    import os
    sys.path.append(os.path.abspath("./../.."))
    from buildz import build
from buildz import xconfz
from buildz import confz

import os
import json
def test(fp = "test.confz"):
    
    obj = xconfz.loads(confz.fread(fp))

    s = confz.dumps(obj, format=True)#, simple =1, as_json = False, t_single = False)
    robj = xconfz.loads(s)
    #robj = json.loads(s)
    return s, obj, json.dumps(robj)

pass

def main():
    nd, obj, robj = test()
    print("obj:")
    print(obj)
    print("nd:")
    print(nd)
    print("robj:")
    print(robj)

pass

if __name__=="__main__":
    main()

pass
"""

python3
import testf
nd, obj = testf.test()

"""



#
from ..ioc.base import Base, EncapeData
from .base import FormatData,BaseDeal
from buildz import xf, pyz
import os
dp = os.path.dirname(__file__)
join = os.path.join
class EnvDeal(BaseDeal):
    def init(self, fp_lists=None, fp_defaults=None):
        super().init("EnvDeal", fp_lists, fp_defaults, join(dp, "conf", "env_lists.js"), None)
    def deal(self, edata:EncapeData):
        data = edata.data
        data = self.fill(data)
        key = data['key']
        return edata.conf.get_env(key)

pass



#coding=utf-8
english = """
1. value:
    1) string
        {
            key: {key}
            val: {value}
        }
    2) int, float, bool
        {
            key: {key}
            val: <{value}, {type}>
        }
    3) from source code:
        {
            key: {key}
            import: {package}
            # var, not val
            var: {var}
        }
    4) value or method from object:
        {
            key: {key}
            ref: {ref}
            # var, not val
            var: {var}
        }
    5) list:
        {
            key: {key}
            args: [
                ...
            ]
        }
    6) dict:
        {
            key: {key}
            # notice this is "[", not "{"
            maps: [
                ...
            ]
        }
    7) just is {data}, not do any transfer
        {
            key: {key}
            data: {data}
        }
2. object:
    {
        # optional, if 0, new every time, else just build a single static object
        single: 0|1
        key: {key}
        import: {import}
        call: {class}
        # optional, should not put with args
        val: {val}
        # optional, should not put with val
        args: [
            ({value}, {type})
            ...
        ]
        # optional
        maps: [
            ({key}, {value}, {type})
            ...
        ]
        # optional, should no with val, args, maps
        data: {data}
        # optional
        calls: [
            ({key}, {type}) #type: call(object.method), fc(static function)
        ]
    }
3. function:
    1) object's method
    {
        key: {key}
        ref: {ref} #optional
        call: {call} #optional, object.{call}(...), if not {call}: object(...)
        val: {val} # optional, should not put with args
        args: [...] # optional, should not put with val
        maps: [...] # optional
        data: {data} # optional, should no with val, args, maps
    }
    # if there is not params in object, put an empty args or empty maps: {key:{key}, ref:{ref}, args:[]}
    2) static function
    {
        key: {key}
        import: {import}
        call: {call}
        val: {val} # optional, should not put with args
        args: [...] # optional, should not put with val
        maps: [...] # optional
        data: {data} # optional, should no with val, args, maps
    }
4. functions:
    {
        key: {key}
        calls: [
            ...
        ]
    }

"""
chinese = """
1. 定义值:
    1) 字符串
        {
            key: {key}
            val: {value}
        }
    2) int, float, bool
        {
            key: {key}
            val: <{value}, {type}>
        }
    3) 源代码中的变量（常量）:
        {
            key: {key}
            import: {package}
            # var, 不是val
            var: {var}
        }
    4) 对象的变量:
        {
            key: {key}
            ref: {ref}
            # var, 不是val
            var: {var}
        }
    5) list:
        {
            key: {key}
            args: [
                ...
            ]
        }
    6) dict:
        {
            key: {key}
            # 注意是"[", 不是"{"
            maps: [
                ...
            ]
        }
    7) 纯数据，不做任何转换
        {
            key: {key}
            data: {data}
        }
2. object对象:
    {
        # 可选，是否是单例
        single: 0|1
        key: {key}
        import: {import}
        call: {class}
        # 可选，不和args一起
        val: {val} 
        # 可选，不和val一起
        args: [
            ({value}, {type})
            ...
        ]
        # 可选
        maps: [
            ({key}, {value}, {type})
            ...
        ]
        # 可选, 不和val, args, maps一起
        data: {data}
        # 可选
        calls: [
            ({key}, {type}) #type: call(object.method), fc(static function)
        ]
    }
3. function:
    1) object's method
    {
        key: {key}
        ref: {ref} #可选
        call: {call} #可选, object.{call}(...), if not {call}: object(...)
        val: {val} # 可选, 不和args一起
        args: [...] # 可选, 不和val一起
        maps: [...] # 可选
        data: {data} # 可选, 不和val, args, maps一起
    # 如果是调用对象本身，并且对象没有任何输入参数，要写一个空的args或maps: {key:{key}, ref:{ref}, args:[]}
    2) static function
    {
        key: {key}
        import: {import}
        call: {call}
        val: {val} # 可选, 不和args一起
        args: [...] # 可选, 不和val一起
        maps: [...] # 可选
        data: {data} # 可选, 不和val, args, maps一起
    }
4. functions:
    {
        key: {key}
        calls: [
            ...
        ]
    }

"""

def help(lang = 'cn'):
    global chinese, english
    lang = lang.lower()
    if lang in ['cn', 'ch']:
        print(chinese)
    else:
        print(english)

pass

"""

{key: demo1, val: "hello world!"}
{key: demo2, val: <100.0, float>}
demo/val.py:
   a = 10
   class X:pass
   b = X()
   b.c = a
{key: demo3, import: demo.val, var: a}
{key: demo3, import: demo.val, var: b.c}



"""




def load(md, fc = None):
    """
        import object(whether module or others) from md(or md.fc)
        exp:
            load("buildz.xf") = package xf
            load("buildz.xf", "loads") = function loads from package buildz.xf
            load("buildz.xf.loads") = function loads from package buildz.xf
    """
    mds = md.split(".")
    arr = mds[1:]
    while len(mds)>0:
        try:
            md = __import__(".".join(mds))
            break
        except:
            mds = mds[:-1]
    if len(mds)==0:
        raise Exception("can't import package from "+md)
    for k in arr:
        md = getattr(md, k)
    if fc is not None:
        fc = getattr(md, fc)
    else:
        fc = md
    return fc

pass
import sys
def pyexe():
    return sys.executable

pass
is_windows = sys.platform.lower()=='win32'
def pypkg():
    """
        return python package path, test on linux and windows
    """
    import site
    sites = site.getsitepackages()
    if is_windows:
        fpath = sites[-1]
    else:
        fpath = sites[0]
    return fpath

pass



#coding=utf-8
from buildz import xf, pyz
from buildz.xf import g as xg
import json
from .base import Base, EncapeData
class Conf(Base):
    def get_key(self, obj, key = 'id', index=0):
        if type(obj)==dict:
            return obj[key]
        id = obj[index]
        if type(id) in [list, tuple]:
            return id[0]
        return id
    def map(self, arr, fc_key):
        return {fc_key(obj): obj for obj in arr}
    def init(self, conf, confs):
        """
            {
                deals:[{build: fc_path,args: [],maps: {}}]
                envs: {id: val}
                id: default null
                namespace: default null
                datas: [{id:val, type: val, ...}]
                locals: [like datas]
                default_type: default null
            }
        """
        id = xf.g(conf, id=None)
        if id is None:
            id = confs.conf_id()
        self.id = id
        self.namespace = xf.g(conf, namespace=None)
        self.conf = conf
        self.confs = confs
        self.locals = self.map(xf.g(conf, locals=[]), self.confs.get_data_id)
        self.datas = self.map(xf.g(conf, datas=[]), self.confs.get_data_id)
        self.deals = self.map(xf.g(conf, deals = []), self.confs.get_deal_type)
        self._default_type = xf.g(conf, default_type = None)
        self.envs = xf.g(conf, envs = {})
        self.confs.flush_env(self.envs)
        for _type in list(self.deals.keys()):
            conf = self.deals[_type]
            if type(conf) in [list, tuple]:
                maps = {}
                maps['type'] = conf[0]
                maps['build'] = conf[1]
                arr = conf[2:]
                if len(arr)>0:
                    maps['args'] = arr.pop(0)
                if len(arr)>0:
                    maps['maps'] = arr.pop(0)
                conf = maps
            fc = pyz.load(conf["build"])
            args = xf.g(conf, args=[])
            maps = xf.g(conf, maps={})
            deal = fc(*args, **maps)
            self.deals[_type] = deal
            aliases = xf.g(conf, aliases = [])
            for alias in aliases:
                self.deals[alias] = deal
    def get_env(self, id, search_confs = True):
        if self.confs.global_env and search_confs:
            return self.confs.get_env(id, self.id)
        ids = self.confs.env_ids(id)
        envs = self.envs
        find = None
        for id in ids:
            if type(envs)!=dict:
                envs = None
                break
            if id not in envs:
                envs = None
                break
            envs = envs[id]
        if envs is not None:
            return envs
        if not search_confs:
            return None
        return self.confs.get_env(id, self.id)
    def get_deal(self, type, search_confs = True):
        if self.confs.global_deal and search_confs:
            return self.confs.get_deal(type, self.id)
        if type in self.deals:
            return self.deals[type]
        if not search_confs:
            return None
        return self.confs.get_deal(type, self.id)
    def get_data(self, id, local = True, search_confs = True):
        if id in self.datas:
            obj = self.datas[id]
            return EncapeData(obj, self, local = False)
        if not local:
            return None
        if id in self.locals:
            obj = self.locals[id]
            return EncapeData(obj, self, local = True)
        if not search_confs:
            return None
        return self.confs.get_data(id, self.id)
    def get(self, *args, **maps):
        return self.get_obj(*args, **maps)
    def default_type(self):
        if self._default_type is None:
            return self.confs.default_type
        return self._default_type
    def get_obj(self, id):
        """
            根据data id获取data对象，处理逻辑：根据data id查配置，根据配置的type查deal，返回deal处理过的配置
        """
        conf = self.get_data(id)
        if conf is None:
            return None
        deal = self.get_deal(conf.type)
        if deal is None:
            return None
        return deal(conf)

pass



#coding=utf-8


"""
impl in class
"""

import io
# io.StringIO("asdf")
from .buff import *
from .fc import *

from .fmt_base import *
from .fmt_type import *
from .mg import *



class StrForamt(BaseFormat):
    def __init__(self, reg, symbol = '"', bts = False, single = True, t_single = False):
        self.symbol = symbol
        self.reg = reg
        self.bts = bts
        self.single = single
        self.t_single = t_single
    def c(self, ch):
        if self.bts and type(ch)==str:
            ch = ch.encode("utf-8")
        return ch
    def f(self, fc, *args, **maps):
        args = [self.c(k) for k in args]
        maps = {k:self.c(maps[k]) for k in maps}
        return fc(*args, **maps)
    def deal(self, node, fc, **params):
        if not node.is_leaf():
            return None
        val = node.value
        if type(val) not in [str, bytes]:
            return None
        if self.single and self.f(val.find, "\n")>=0:
            return None
        if not self.reg.exist(val) and val.strip()!="":
            return val
        val = self.f(val.replace, "\\", "\\\\")
        if self.t_single:
            val = self.f(val.replace, "\n", "\\n")
        val = self.f(val.replace, "\"", "\\\"")
        return self.symbol+val+self.symbol

pass




import xf
s=r"""
{
    a:b,
    test: ???
    id: 123
    cid: '123'
    content: '''hehe
    hehehe
    '''
    b:,#test
    a:[1,2,3,]
    v:{
        a:r"xx\n"
        b:"\n"
        c:d
        url:"http://test.com.cn"
    }
    # test
    hehe: <"1", i>
    nill: <,nil>
}
"""
rst = xf.loads(s)
print(rst)


"""
import sys

sys.path.append(r"")

"""
try:
    from buildz import build
except:
    import sys
    import os
    sys.path.append(os.path.abspath("./../.."))
    from buildz import build

build.test()




from buildz import xconfz as xfz
import xf
import json 
s = r"""
{
    a:b,
    test: ???
    id: 123
    cid: '123' # nothing here
    /*
    >>>>>>>
    */
    content: '''hehe
    hehehe
    '''
    b:xx,
    int: 1
    null: null
    a:[1,2,3,]
    v:{
        a:r"xx\n"
        b:"\n"
        c:d
        url:"http://test.com.cn"
    }
}
"""
def test():
    obj = xfz.loads(s)
    obj = xf.loads(s)
    rs = xf.dumps(obj,format=1,deep=1,json_format=1)
    print(obj)
    print(json.dumps(obj))
    print(rs)

pass
test()


r"""

cd D:\rootz\python\gits\buildz_upd\buildz\xf

python test_write.py



"""


#coding=utf-8
"""
一些简单工具代码
"""

"""
a = [rp("<", ">")(script, "id", k[0], "val", k[1]) for k in a]

"""
class rp:
    def __init__(self, l="<", r=">"):
        self.l = l
        self.r = r
    def __call__(self, s, *params):
        for i in range(0, len(params), 2):
            s = s.replace(self.l+params[i*2]+self.r, params[i*2+1])
        return s

pass


import os
def listdirs(fp, maxdepth = -1, include_dir = False, abs = True):
    if abs:
        fp = os.path.abspath(fp)
    if not os.path.exists(fp):
        print("error path:", fp)
        return []
    if os.path.isfile(fp):
        return [fp]
    fs = os.listdir(fp)
    fs = [os.path.join(fp, k) for k in fs]
    rst = []
    for f in fs:
        rst += listdirs(f, maxdepth-1, include_dir, abs)
    return rst

pass




#coding=utf-8

from . import dirz

import re
class ListsDeal(dirz.FileDeal):
    def result(self):
        return self.fps, self.errors
    def init(self):
        super().init()
        self.fps = []
        self.errors = []
    def visit(self, fp, isdir, depth):
        self.fps.append([fp, isdir])
        return True
    def catch(self, fp, isdir, depth, exp):
        self.errors.append([fp, isdir, exp])

pass

class FileSearchDeal(dirz.FileDeal):
    def init(self, pt_fp=None, pt = None):
        super().init()
        self.pt_fp = pt_fp
        if type(pt) == str:
            pt = pt.encode()
        self.pt = pt
        self.rst = []
    def result(self):
        return self.rst
    def visit(self, filepath, isdir, depth):
        if isdir:
            return True
        if self.pt_fp is not None and len(re.findall(self.pt_fp, filepath))==0:
            return False
        if self.pt is not None:
            try:
                with open(filepath, 'rb') as f:
                    s = f.read()
            except Exception as exp:
                print("fread exp in :", filepath, "exp:", exp)
                return False
            if len(re.findall(self.pt, s))==0:
                return False
        #print("find:", filepath)
        self.rst.append(filepath)
        return False
    def catch(self, filepath, isdir, depth, exp):
        print(f"exp in {filepath} isdir={isdir}: {exp}")
        pass

pass


def lists(fp):
    return ListsDeal().dirs(fp)

pass
def search(dp, pt_fp = None, pt = None):
    return FileSearchDeal(pt_fp, pt).dirs(dp)

pass



#coding=utf-8

from .base import *

class BufferImpl(Buffer):
    def cal_pos(self, s):
        n = "\n"
        if type(s)==bytes:
            n = b"\n"
        arr= s.split(n)
        return len(arr)-1, len(arr[-1])
    def __init__(self, reader, empty = ""):
        self._read = reader
        self._remain = empty
        self._curr = empty
        self.index = [0,0]
        self.row = [0,0]
        self._empty= empty
    def pos_remain(self):
        return self.row[0]+1, self.index[0]+1
    def pos_curr(self):
        return self.row[1]+1, self.index[1]+1
    def curr(self, size = 1):
        if self._curr is None:
            self._curr = self._read(size)
        elif len(self._curr)<size:
            ch = self._read(size-len(self._curr))
            if len(ch)==0:
                return self._curr
            self._curr += ch
        return self._curr[:size]
    def remain_size(self):
        return 0 if self._remain is None else len(self._remain)
    def remain(self):
        return self._remain
    def deal_remain(self):
        self.index[0] = self.index[1]
        self.row[0] = self.row[1]
        self._remain= self._empty
    def add_remain(self,size=1):
        curr = self.curr(size)
        if len(curr)<size:
            return False
        row, id = self.cal_pos(curr)
        if row == 0:
            self.index[1]+=id
        else:
            self.row[1]+=row
            self.index[1]= id
        self._remain+=curr
        self._curr = self._curr[size:]
        return True
    def deal2curr(self, size = 1):
        bl = self.add_remain(size)
        self.deal_remain()

pass



#coding=utf-8
from .base import *

from .fmt_base import *

"""
去左空格
"""
class SpcDeal(BaseDeal):
    def prev(self, buff, stack):
        if buff.remain_size()==0:
            c = buff.curr()
            if len(c.strip())==0 and len(c)>0:
                buff.deal2curr()
                return True
        return False

pass

"""
没有其他处理的默认处理：当前字符放入保留字符串，读取下一个字符
该处理应该放处理列表的最后（优先级最低）
"""
class NextCharDeal(BaseDeal):
    def prev(self, buff, queue):
        #print("SRC:", buff.remain()," : ", buff.curr(100))
        rst= buff.add_remain()
        #print("ADD:", buff.remain(), " : " ,buff.curr(100))
        return rst
    def deal(self, queue, stack):
        if len(queue)==0:
            return False
        stack.append(queue.pop(0))
        return True

pass

"""
分割符判断
"""
class ItemDeal(BaseDeal):
    def find(self, buff):
        for spt in self.spts:
            if self.check_curr(buff, spt):
                return spt
        return None
    def __init__(self, *args):
        if len(args)==0:
            raise Exception("at least one symbol")
        self.spts= args
    def init(self, reg):
        self.k_spts = [reg(k) for k in self.spts]
        self.k_spt = self.k_spts[0]
    def prev(self, buff, queue):
        find = self.find(buff)
        if find is None:
            return False
        lf = len(find)
        rm = buff.remain().strip()
        pos = buff.pos_remain()
        c_pos = buff.pos_curr()
        if len(queue)==0 or len(rm)>0 or self.k_spt(queue[-1].val):
            #print("val:[{rm}]".format(rm=rm))
            queue.append(Item(rm, pos))
        queue.append(Item(self.k_spt, c_pos, find))
        buff.deal2curr(lf)
        return True
    def deal(self, queue, stack):
        if len(queue)==0:
            return False
        it = queue[0]
        if self.k_spt(it.val):
            queue.pop(0)
            return True
        return False

pass

'''
数据生成格式化字符串
'''
class ItemFormat(BaseFormat):
    def deal(self, data, fc):
        node = FormatNode().init().val(data)
        return node


pass


#coding=utf-8
class Demo:
    def __init__(self, a, b):
        self.a = a
        self.b = b
    def show(self, data):
        print("START SHOW")
        print("self:", self.a, self.b)
        print("input:", data)
        print("FINISH SHOW")
    def __call__(self, data):
        print("CALL __call__ that do nothing")
    def __str__(self):
        return "<Object Demo a=("+str(self.a)+"), b=("+str(self.b)+")>"
    def __repr__(self):
        return str(self)

pass

demo_value = "hello world!"



#coding=utf-8
import sys
def fetch(argv = None):
    r"""
    format: a b c -a 123 -b456 --c=789 +d  -x"??? ???" y z
    return: [a,b,c,y,z], {a:123,b:456,c:789,d:1,x:'??? ???'}
    """
    if argv is None:
        argv = sys.argv[1:]
    lists, maps = [],{}
    argv = [str(k).strip() for k in argv]
    argv = [k for k in argv if k!=""]
    i = 0
    while i<len(argv):
        v = argv[i]
        if v in ["-", "--", "+"]or v[0] not in "+-":
            lists.append(v)
            i+=1
            continue
        if v[0] == "+":
            key = v[1:]
            val = 1
        else:
            if v[1]=="-":
                kv = v[2:]
                x = kv.split("=")
                key = x[0]
                val = "=".join(x[1:])
                if len(val)==0:
                    val = 1
            else:
                key = v[1]
                if len(v)>2:
                    val = argv[2:]
                else:
                    if i+1>=len(argv):
                        val = 1
                    else:
                        val = argv[i+1]
                        i+=1
        if key not in maps:
            maps[key] = []
        maps[key].append(val)
        i+=1
    for k in maps:
        v = maps[k]
        if len(v)==1:
            maps[k] = v[0]
    return lists, maps

pass

        




#coding=utf-8


"""
impl in class
"""

import io
# io.StringIO("asdf")
from .buff import *
from .fc import *

from .fmt_base import *
from .fmt_type import *
from .fmt_str import *
from .mg import *

class Buildz:
    def __init__(self, bts= False):
        self.bts= bts
        self.deals = {}
        self._default = None
    def default(self, _def):
        self._default = _def
    def fcs(self, fc, *args, **maps):
        return fc(*self.cs(*args), **maps)
    def set_fc(self, type, fc, *args, **maps):
        obj = self.fcs(fc, *args, **maps)
        self.set(type, obj)
        return self
    def cs(self, *args):
        return [self.c(k) for k in args]
    def c(self, s):
        if self.bts:
            s = s.encode("utf-8")
        return s
    def set(self, type, fc):
        self.deals[type] = fc
    def __call__(self, data):
        tp = type(data)
        if tp not in self.deals:
            return self._default(data, self)
        return self.deals[tp](data, self)

pass

class Outputz(Confz):
    def __init__(self, bts= False):
        self.bts= bts
        self.deals = []
    def add(self,obj):
        self.deals.append(obj)
        return self
    def add_top(self, obj):
        self.deals=[obj]+self.deals
        return self
    def __call__(self, data, **params):
        for deal in self.deals:
            val = deal(data, self, **params)
            if val is not None:
                return val
        return None

pass
class Formatz:
    def __init__(self, bts = False):
        self.build = Buildz(bts)
        self.output = Outputz(bts)
    def __call__(self, data, **params):
        node = self.build(data)
        rst = self.output(node, **params)
        return rst

pass
class JsonFormat(BaseFormat):
    def deal(self, node, fc, **params):
        import json
        if not node.is_leaf():
            return None
        value = node.value
        if value is None:
            return None
        return json.dumps(node.value, ensure_ascii=False)

pass
def build_format(bts = False, size = 4, format = False, simple = 1, simple_size = 1, loads = None, as_json =False, t_single = True):
    if loads is None:
        loads = build()
    fmtz = Formatz(bts)
    fmtz.build.default(ItemFormat())
    fmtz.build.set_fc(list, ListFormat,"[", "]")
    fmtz.build.set_fc(tuple, ListFormat,"(", ")")
    fmtz.build.set_fc(dict, MapFormat,"{","}")


    types = TypesDealFormat()
    #types.add_deal(fmtz.output.fcs(TypeDealBFormat, "|"))
    types.add_deal(fmtz.output.fcs(TypeDealAFormat, "<", ">", ","))

    types.add_type(int, lambda x:[str(x), "i"])
    types.add_type(float, lambda x:[str(x), "f"])
    types.add_type(str, lambda x:[str(x), "s"])
    types.add_type(bool, lambda x:[str(x), "b"])
    types.add_type(type(None), lambda x:[str(x), "nil"])
    types.init()

    fmtz.output.add_fc(NodeFormat)
    fmtz.output.add_fc(SymbolFormat)
    spt = ","
    if not format:
        simple_size = 0
        spt = ", "
    fmtz.output.add_fc(KeyFormat, spt, SptFormatNode)
    fmtz.output.add_fc(KeyFormat, ": ", KVFormatNode)
    fmtz.output.add_fc(SpcFormat, " ", "\n", size = size, format = format, simple = simple, simple_size = simple_size)
    if as_json:
        fmtz.output.add_fc(JsonFormat)
    fmtz.output.add_fc(StrForamt, loads.reg, '"', bts, not t_single, t_single)
    fmtz.output.add_fc(StrForamt, loads.reg, '"""', bts, False)
    fmtz.output.add(types)
    return fmtz

pass
def dumps(data, bts = False, size = 4, format = False, simple = 1, simple_size = 1, as_json =False, t_single = True):
    fmtz = build_format(bts=bts, size = size, format = format, simple = simple, simple_size = simple_size, as_json =as_json, t_single =t_single)
    nd = fmtz(data)
    return nd

pass

class Map:
    def __init__(self):
        self.maps = {}
    def set(self, key, val):
        self.maps[key] = val
    def __call__(self, s):
        if s not in self.maps:
            raise Exception("Error key:"+s)
        return self.maps[s]

pass
def get_none(s):
    return None

pass



#coding=utf-8
from .base import *


class StrDeal(BaseDeal):
    def __init__(self, left = '"', right= '"', single_line = False, note = False):
        self.left = left
        self.right = right
        self.single_line = single_line
        self.note = note
    def init(self,reg):
        self.k_l = reg(self.left)
        if self.left == self.right:
            self.k_r = self.k_l
        else:
            self.k_r = reg(self.right)
    def prev(self, buff, queue):
        if not self.check_curr(buff, self.left):
            return False
        rm = buff.remain().strip()
        if len(rm)>0 and not self.note:
            raise FormatExp("str: unexcept char before symbol:", buff.pos_remain(), rm)
        left_pos = buff.pos_curr()
        buff.deal2curr(len(self.left))
        lr = len(self.right)
        curr = buff.curr(lr)
        while len(curr)>=lr:
            if curr == self.right:
                s = buff.remain()
                if not self.note:
                    queue.append(Item(s, left_pos))
                elif len(rm)>0:
                    queue.append(Item(rm, left_pos))
                buff.deal2curr(lr)
                return True
            if self.single_line and curr.find("\n")>=0:
                raise FormatExp("str|note: unexcept enter for string", buff.pos_remain())
            buff.add_remain(1)
            curr = buff.curr(lr)
        print(buff.remain())
        err = "str|note: can't find right symbol {right} for left symbol {left}".format(right =  self.right, left = self.left)
        raise FormatExp(err, left_pos)

pass

class TranslateStrDeal(StrDeal):
    def prev(self, buff, queue):
        if not self.check_curr(buff, self.left):
            return False
        rm = buff.remain().strip()
        if len(rm)>0:
            raise FormatExp("str|note: unexcept char before symbol:", buff.pos_remain())
        left_pos = buff.pos_curr()
        buff.deal2curr(len(self.left))
        lr = len(self.right)
        cnt = 0
        curr = buff.curr(1)
        mark = False
        while len(curr)>0:
            cnt += 1
            if curr in ["\\", b"\\"] and not mark:
                mark = True
            else:
                rm += curr
                if mark:
                    cnt = 0
                mark = False
            if not mark and cnt >= lr:
                if rm[-lr:] == self.right:
                    rm = rm[:-lr]
                    if not self.note:
                        queue.append(Item(rm, left_pos))
                    buff.deal2curr(1)
                    return True
            buff.add_remain(1)
            curr = buff.curr(1)
        err = "str|note: can't find right symbol {right} for left symbol {left}".format(right =  self.right, left = self.left)
        raise FormatExp(err, left_pos)

pass


#coding=utf-8
from .build import test
from . import keys
import sys
def _main():
    v = sys.argv[1]
    lang = 'ch'
    if v in ['?', '-h', '--help']:
        if len(sys.argv)>2:
            lang = sys.argv[2].strip().lower()
        keys.help(lang)
    else:
        test()

pass
if __name__=="__main__":
    _main()

pass



#coding=utf-8
import builtins
import os
str = builtins.str
int = builtins.int
float = builtins.float
str = builtins.str
class Map:
    Key = object()
    def __dir__(self):
        return dir(Map)
    def __init__(self, obj = None, **maps):
        if obj is None:
            obj = {}
        else:
            obj = builtins.dict(obj)
        obj.update(maps)
        self.__dict__[Map.Key] = obj
    def __call__(self, **maps):
        if len(maps)==0:
            return self.__dict__[Map.Key]
        self().update(maps)
        return self
    def __getattr__(self, key):
        if key not in self():
            return None
        return self()[key]
    def __getitem__(self, key):
        if key not in self():
            return None
        return self()[key]
    def __setitem__(self, key, val):
        maps = self()
        maps[key] = val
    def __setattr__(self, key, val):
        maps = self()
        maps[key] = val

pass

def list(*argv):
    return builtins.list(argv)

pass

def dict(**maps):
    return builtins.dict(maps)

pass
def val(v):
    return v

pass

def add(*argv):
    rst = argv[0]
    for v in argv[1:]:
        rst += v
    return rst

pass

def join(*argv):
    return os.path.join(*argv)

pass


class SimpleDeal:
    def deal(self, filepath, isdir, depth):
        return True

pass

def listdir(path, deal = None, depth = 0):
    files = os.listdir(path)
    files = [os.path.join(path, file) for file in files]
    rst = []
    for file in files:
        if os.path.isdir(file):
            if deal(file, True, depth+1):
                listdir(file, deal, depth+1)
        else:
            deal(file, False, depth+1)
    return rst

pass




#
from buildz.ioc import build
from buildz import xf
from buildz import argx
import os
join = os.path.join
class Test:
    def __init__(self, *args, **maps):
        print(f"Test init args: {args}, maps: {maps}")
        self.args = args
        self.maps = maps
    def run(self):
        print(f"get obj:{self}")
        print("get_obj.obj:", self.obj)
        print(f"ioc: {self.ioc}")
        print(f"ioc env: {self.ioc.get_env('path')}")

pass
fps = xf.loads("[data1.js, data2.js, data3.js]")[:2]

fps = [join('test_conf', fp) for fp in fps]
def show():
    print("run show")

pass
def test():
    confs = build()
    for fp in fps:
        confs.add_fp(fp)
    #obj =  confs.get("data1.test")
    confs.get("data1.run")
    obj = confs.get("data1.test.obj")
    print(f"obj: {obj}")
    print("DO CALLS")
    confs.get("data1.calls")
    print("ARGS: ", argx.fetch())

pass

if __name__=="__main__":
    test()

pass


#coding=utf-8
#main
def get(obj, key, default = None):
    if key in obj:
        return obj[key]
    return default

pass
def load(md, fc = None):
    arr = md.split(".")[1:]
    md = __import__(md)
    for k in arr:
        md = getattr(md, k)
    if fc is not None:
        fc = getattr(md, fc)
    else:
        fc = md
    return fc

pass
class EpFc:
    def __init__(self, fc, n = 1):
        self.fc = fc
        self.n = n
    def __call__(self, *argv):
        argv = argv[:self.n]
        return self.fc(*argv)

pass
import builtins
def build_list(*argv):
    return list(argv)

pass

def build_dict(**maps):
    return dict(maps)

pass
builtins._list = build_list
builtins._dict = build_dict
g_default_import = "buildz"+".base"
def read_confz(filepath):
    from . import confz
    s = confz.fread(filepath)
    obj = confz.read(s)
    return obj

pass
def read_json(filepath):
    from . import confz
    import json
    s = confz.fread(filepath)
    obj = json.loads(s)
    return obj

pass
    
class Builder:
    def __init__(self, default_single = "0", default_import = g_default_import, ref_this = None, format = "confz"):
        self.ref_this = ref_this
        self.default_import = default_import
        self.default_single = default_single
        self.format = format
        self.maps = {}
        self.objs = {}
        types = {}
        types['str'] = EpFc(str)
        types['int'] = EpFc(int)
        types['float'] = EpFc(float)
        self.types = types
        self.types['ref'] = EpFc(self.run)
        self.types['this'] = EpFc(self.get_this, 2)
        self.types['call'] = EpFc(self.run, 2)
        self.types['fc'] = EpFc(self.run, 1)
        self.types['run'] = EpFc(self.run, 1)
        self.loadtypes = {}
        self.loadtypes['json'] = read_json
        self.loadtypes['confz'] = read_confz
    def add_file(self, filepath, format = None, update = False):
        if format is None:
            format = self.format
        obj = self.loadtypes[format](filepath)
        self.add(obj, update)
    def add(self, data, update = False):
        if type(data) == dict:
            data = [data]
        for obj in data:
            key = get(obj, "key")
            if key in self.maps and not update:
                raise Exception("duple define key: ["+key+"]")
            self.maps[key] = obj
            if update and key in self.objs:
                del self.objs[key]
    def get_this(self, key, obj):
        return getattr(obj, key)
    def deal_args(self, args, obj = None):
        if type(args)!= list:
            args = [args]
        rst = []
        for _obj in args:
            if type(_obj) not in [list, tuple]:
                _obj = [_obj, "str"]
            val = self.types[_obj[1]](_obj[0], obj)
            rst.append(val)
        return rst
    def deal_maps(self, maps, obj = None):
        rst = {}
        for _obj in maps:
            if len(_obj)==2:
                _obj.append("str")
            _key = _obj[0]
            if type(_key) == list:
                _key = self.run(_key)
            val = self.types[_obj[2]](_obj[1], obj)
            rst[_key] = val
        return rst
    def deal_sets(self, sets, obj):
        for param in sets:
            if len(param)==2:
                param.append("str")
            _key = param[0]
            val = self.types[param[2]](param[1], obj)
            setattr(obj, _key, val)
    def deal_fcs(self, calls, obj):
        for call in calls:
            if type(call)!= list:
                call = [call, "call"]
            self.types[call[1]](call[0], obj)
    def get(self, key, src = None, force_new = False):
        return self.run(key, src, force_new)
    def set(self, key, val):
        self.maps[key] = {"single":"1"}
        self.objs[key] = val
    def run(self, key, src = None, force_new = False):
        if self.ref_this is not None and key == self.ref_this:
            return self
        if key not in self.maps:
            raise Exception("Error not such fc:"+key)
        data = self.maps[key]
        ref = get(data, "ref")
        if src is None and ref is not None:
            src = self.run(ref)
        if src is not None:
            force_new = True
        single = bool(int(get(data, "single", self.default_single)))
        single &= not force_new
        if single and key in self.objs:
            return self.objs[key]
        val = get(data, "val", None)
        if val is not None:
            args = [val]
        else:
            args = get(data, "args", [])
        args = self.deal_args(args, src)
        maps = get(data, "maps", [])
        maps = self.deal_maps(maps, src)
        fc = get(data, "call", None)
        var = get(data, "var", None)
        mark_var = False
        if fc is None:
            if var is not None:
                mark_var = True
                fc = var
        obj = None
        _data = get(data, "data", None)
        if fc is not None or src is not None:
            fcs = []
            if fc is not None:
                fcs = fc.split(".")
            if src is not None:
                fc = src
            else:
                _import = get(data, "import", self.default_import)
                md = load(_import)
                fc = md
            for nfc in fcs:
                fc = getattr(fc, nfc)
            if mark_var:
                return fc
            if _data is None:
                obj = fc(*args, **maps)
            else:
                obj = fc(_data)
            if single:
                self.objs[key] = obj
            sets = get(data, "sets", [])
            self.deal_sets(sets, obj)
        calls = get(data, "calls", [])
        self.deal_fcs(calls, obj)
        if obj is None:
            if _data is not None:
                return _data
            elif val is not None:
                return val
            elif args is not None:
                return args
            elif maps is not None:
                return maps
        return obj

pass
class Test:
    def __init__(self, a, b):
        print("a:", a)
        self.a = a
        self.b = b
    def test(self, obj):
        print("TEST:", obj)
    def __call__(self):
        print(self.a)
        print(self.b)

pass

import sys
import os
def main(paths, default_import = g_default_import, ref_this = "this"):
    builder = Builder(1, default_import = default_import, ref_this = ref_this)
    for path in paths:
        if os.path.isfile(path):
            builder.add_file(path)
        elif os.path.isdir(path):
            files = os.listdir(path)
            files = [os.path.join(path, file) for file in files]
            [builder.add_file(file) for file in files]
        else:
            print("what is this? :", path)
    builder.run("main")

pass
import os
import re
class Pattern:
    def __init__(self, pt):
        self.pt = pt
    def __call__(self, s):
        if self.pt is None:
            return True
        return re.findall(self.pt, s)>0

pass


def listdir(path, max_depth = -1, pattern = None, nopattern = None, march = None):
    if max_depth == 0:
        return []
    files = os.listdir(path)
    pt = Pattern(pattern)
    files = [os.path.join(path, file) for file in files]
    rst = []
    for file in files:
        if os.path.isdir(file):
            rst += listdir(file, max_depth-1, pattern)
        else:
            if pt(os.path.basename(file)):
                rst.append(file)
    return rst

pass

    
def test():
    builder = Builder(1)
    argv = sys.argv[1:]
    main(argv)

pass

if __name__=="__main__":
    test()

pass




#coding=utf-8

from ..base import Map
'''
格式化报错
'''
class FormatExp(Exception):
    def __init__(self, err, data, s = ""):
        if len(s)==0:
            errs = "Error: {err}, line: {line}, index: {index}".format(err = err, line = data[0], index = data[1])
        else:
            errs = "Error: {err}, line: {line}, index: {index}, content: [{s}]".format(err = err, line = data[0], index = data[1], s = s)
        super(FormatExp, self).__init__(errs)

pass

'''
数据封装类
val是数据（字符）值
pos是字符在文件里的位置(第几行第几列)
remain和type？
'''
class Item:
    def __str__(self):
        return "<item val={val}, type = {type}, pos = {pos}>".format(val = str(self.val), type = str(self.type), pos = str(self.pos))
    def __repr__(self):
        return str(self)
    def __init__(self, val, pos, remain=None, type = None):
        self.val= val
        self.pos= pos
        self.remain= remain
        self.type = type

pass

"""
1,使用框架
2，字符串+index
文本缓存类
位置都是（行，列）
方法名写的有点乱
remain是未处理数据，curr是当前数据，未处理数据在当前数据之前
一种弱约束，remain里的应该当作数据，curr里用来判断是否是符号，不是则加到remain里
"""
class Buffer:
    def pos_remain(self):
        '''
        前置指针在字符串的位置
        '''
        return 0,0
    def pos_curr(self):
        '''
        当前指针在字符串的位置
        '''
        return 0,0
    def pos(self):
        '''
        不记得这是啥
        '''
        return 0,0
    def curr(self, size = 1):
        '''
        返回当前指针开始，长度为size的字符串
        '''
        return ""
    def remain_size(self):
        '''
        前置指针到当前指针的数据量
        '''
        return 0
    def remain(self):
        '''
        返回前置数据
        '''
        return ""
    def full(self, size = 1):
        '''
        前置+当前数据
        '''
        return self.remain()+self.curr(size)
    def deal_remain(self):
        '''
        调用说明前置数据已经处理，前置指针移到当前指针位置
        '''
        pass
    def deal2curr(self, size=1):
        '''
        读取size字节数据，都当作已经处理（包括前置里的数据，当前指针后移size字节，前置指针移到当前指针位置）
        '''
        pass
    def add_remain(self,size=1):
        '''
        当前指针往后移，提取出来的数据放到前置中，如果剩余数据数量不到size，返回false
        '''
        return False

pass

'''
'''
def is_key(val):
    return isinstance(val, Key)

pass

'''
封装符号类
c是符号值
params是配置的其他参数，用来deal里做特殊判断或处理
'''
class Key:
    @staticmethod
    def is_inst(val):
        return isinstance(val, Key)
    def __init__(self, c, params=None):
        self.c = c
        if params is None:
            params = {}
        self.params = Map(params)
        self._params = self.params()
    @property
    def p(self):
        return self.params
    def get(self, key):
        if key not in self._params:
            return None
        return self.params[key]
    def __str__(self):
        return "[key "+str(self.c)+"]"
    def __eq__(self, obj):
        return self(obj)
    def __repr__(self):
        return str(self)
    def equal(self, obj):
        return self(obj)
    def __call__(self, obj):
        if type(obj)!= Key:
            return False
        return obj.c == self.c

pass

'''
符号-数值类
'''
class KeyVal:
    def __repr__(self):
        return str(self)
    def __str__(self):
        return "[keyval {key}:{val}]".format(key = str(self.key), val = str(self.val))
    @staticmethod
    def is_inst(val):
        return isinstance(val, KeyVal)
    def __init__(self, key, val):
        self.key = key
        self.val= val

pass

'''
符号注册，
同一个符号可以多次注册，但后面的注册返回的是最早注册生成的对象
'''
class Reg:
    def __init__(self):
        self.index = 0
        self.sets = set()
        self.keys = {}
    def exist(self, s):
        bts = type(s)==bytes
        for key in self.keys:
            val = key
            if bts:
                val = val.encode("utf-8")
            if s.find(val)>=0:
                return True
        return False
    def __call__(self, key, **params):
        if key in self.keys:
            return self.keys[key]
        #if key in self.keys:
        #    raise Exception("reg key already regist:"+str(key))
        self.keys[key] = Key(key, params)
        return self.keys[key]

pass

'''
字符串处理，分成两个阶段
一阶段prev，生成数值和符号封装对象放到列表里
二阶段deal，从列表里拿符号和数值，生成实际数据
'''
class BaseDeal:
    def check_curr(self, buff,  s):
        '''
        判断缓存当前指针字符串是否和字符串s相同
        '''
        return buff.curr(len(s))==s
    def init(self, reg):
        '''
        初始化，deal放到实际调用的场景时调用，reg用来注册符号
        '''
        pass
    def make(self, data, fc, format = False, simple = True):
        '''
        本来打算数据生成字符串用的，实际没用到
        '''
        return None
    def prev(self, buff, queue):
        '''
        处理缓存buff里的数据，放到列表queue里
        返回false说明没做处理，换下一个deal处理
        '''
        return False
    def deal(self, queue, stack):
        '''
        处理列表queue里的数据，放到堆栈stack里
        返回false说明没做处理，换下一个deal处理
        '''
        return False

pass



from buildz import xf

from os.path import join, dirname
dp = dirname(__file__)
fp = join(dp, "test.js")

s = xf.fread(fp)
obj = xf.loads(s)

rs = xf.dumps(obj, format=1, deep = 1)
print(rs)
args = xf.args()
print("args from cmd:", args)




from writer import mg, itemz, base, conf
from writer.deal import listz, mapz, strz, reval
pts = [
    "[\+\-]?\d+",
    "[\+\-]?\d+\.\d+",
    "[\+\-]?\d+e[\+\-]?\d+",
    "null",
    "true",
    "false",
    ".*[\n\r\t\:\[\]\{\}].*"
]
def build():
    mgs = mg.Manager()
    mgs.add(strz.StrDeal('"','"', pts))
    mgs.add(reval.ValDeal(float, lambda x:str(x)))
    mgs.add(reval.ValDeal(int, lambda x:str(x)))
    mgs.add(reval.ValDeal(type(None), lambda x:'null'))
    mgs.add(reval.ValDeal(bool, lambda x:'true' if x else 'false'))
    mgs.add(listz.ListDeal('[',']',','))
    mgs.add(mapz.MapDeal('{','}',':',','))
    return mgs

pass
def dumps(obj):
    cf = conf.Conf()
    cf.set(bytes=0, format=1, deep=0)
    mgs = build()
    return mgs.dump(obj, cf)

pass

from buildz import xconfz as xf
s = r"""
{
    a:b,
    test: ???
    id: 123
    cid: '123'
    content: '''hehe
    hehehe
    '''
    b:xx,
    int: <1,i>
    null: <null, null>
    a:[1,2,3,]
    v:{
        a:r"xx\n"
        b:"\n"
        c:d
        url:"http://test.com.cn"
    }
}
"""
def test():
    obj = xf.loads(s)
    rs = dumps(obj)
    print(obj)
    print(rs)

pass
test()


r"""

cd D:\rootz\python\gits\buildz_upd\buildz\xf

python test_write.py



"""


#coding=utf-8
from .base import *

from .fmt_base import *

class MapDeal(BaseDeal):
    def init(self, reg):
        self.k_l = reg(self.l)
        self.k_r = reg(self.r)
    def __init__(self, left, right):
        self.l = left
        self.r = right
    def prev(self, buff, queue):
        if self.check_curr(buff,self.l):
            if buff.remain_size()>0:
                raise FormatExp("error string before map:", buff.pos_curr(), buff.full())
            queue.append(Item(self.k_l, buff.pos_curr()))
            buff.deal2curr(len(self.l))
            return True
        elif self.check_curr(buff, self.r):
            if buff.remain_size()>0:
                r = buff.remain().strip()
                if len(r)>0:
                    queue.append(Item(r, buff.pos_remain()))
            queue.append(Item(self.k_r, buff.pos_curr()))
            buff.deal2curr(len(self.r))
            return True
        return False
    def deal(self, queue, stack):
        if len(queue)==0:
            return False
        it = queue[0]
        rst = False
        if self.k_r.equal(it.val):
            tmp = []
            find_l = False
            while len(stack)>0:
                it_1 =  stack.pop(-1)
                if self.k_l.equal(it_1.val):
                    find_l = True
                    break
                val = it_1.val
                if not KeyVal.is_inst(val):
                    #print("it_1:", it_1)
                    #print("queue:", queue)
                    #print("stack:", stack)
                    #print("map:", tmp)
                    raise FormatExp("an not key-val item found in map:"+str(val), it_1.pos)
                tmp.append(val)
            tmp.reverse()
            tmp = {k.key:k.val for k in tmp}
            #tmp[val.key] = val.val
            if not find_l:
                raise FormatExp("can't find map left side for right side",it.pos)
            stack.append(Item(tmp, it_1.pos))
            rst = True
        elif self.k_l.equal(it.val):
            stack.append(it)
            rst = True
        if rst:
            queue.pop(0)
        return rst

pass

class MapFormat(BaseFormat):
    def __init__(self, left, right):
        self.l = left
        self.r = right 
    def deal(self, data, fc):
        if type(data) !=dict:
            raise FormatExp("not dict found:"+type(data), [-1,-1])
        node = FormatNode().init()
        node.add(SymbolFormatNode().init().val(self.l))
        #_node = FormatNode().init()
        #node.add(_node)
        nds = []
        for key in data:
            nds.append(SpcFormatNode(0,1))
            nds.append(fc(key))
            nds.append(KVFormatNode())
            nds.append(fc(data[key]))
            nds.append(SptFormatNode())
        for nd in nds[:-1]:
            node.add(nd)
        node.add(SpcFormatNode(0, 0))
        node.add(SymbolFormatNode().init().val(self.r))
        return node


pass


#coding=utf-8
from .base import *
from .fc_list import *
from .fc_set import *

from .fmt_base import *

"""
二元符<val, type>

"""

class TypeDealAFormat(BaseFormat):
    def __init__(self, left, right, spt):
        self.left = left
        self.right = right
        self.spt = spt
        self._types = {}
    def types(self, maps):
        self._types = dict(maps)
    def deal(self, node, fc, **params):
        if not node.is_leaf():
            return None
        val = node.value
        _type = type(val)
        if _type not in self._types:
            return None
        _fc = self._types[_type]
        sval, stype = _fc(val)
        spt = self.spt
        s = self.left+sval+spt+stype+self.right
        return s

pass

"""
二元符val|type
"""
class TypeDealBFormat(BaseFormat):
    def __init__(self, spt):
        self.spt = spt
        self._types = {}
    def types(self, maps):
        self._types = dict(maps)
    def deal(self, node, fc, **params):
        if not node.is_leaf():
            return None
        val = node.value
        _type = type(val)
        if _type not in self._types:
            return None
        _fc = self._types[_type]
        sval, stype = _fc(val)
        return sval+self.spt+stype

pass


class TypesDealFormat(BaseFormat):
    def __init__(self):
        self.deals = []
        self.types = {}
    def add_deal(self, deal):
        self.deals.append(deal)
        return self
    def add_type(self, key, fc):
        self.types[key] = fc
        return self
    def add(self, obj, fc = None):
        if isinstance(obj, BaseDeal):
            return self.add_deal(obj)
        else:
            return self.add_type(obj, fc)
    def init(self):
        for deal in self.deals:
            deal.types(self.types)
    def deal(self, node, fc, **params):
        for deal in self.deals:
            obj = deal.deal(node, fc, **params)
            if obj is not None:
                return obj
        return None

pass


